#!/usr/bin/python
#-*- coding: utf-8 -*-

"""
SAT communication bridge
Copyright (C) 2009, 2010, 2011  Jérôme Poisson (goffi@goffi.org)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from bridge_frontend import BridgeFrontend
import dbus, dbus.glib
from logging import debug, error

const_INT_PREFIX = "org.goffi.SAT"  #Interface prefix
const_OBJ_PATH = '/org/goffi/SAT/bridge'
const_CORE_SUFFIX = ".core"
const_PLUGIN_SUFFIX = ".plugin"

class BridgeExceptionNoService(Exception):
    pass

class DBusBridgeFrontend(BridgeFrontend):
    def __init__(self):
        try:
            self.sessions_bus = dbus.SessionBus()
            self.db_object = self.sessions_bus.get_object(const_INT_PREFIX,
                                const_OBJ_PATH)
            self.db_core_iface = dbus.Interface(self.db_object,
                                dbus_interface=const_INT_PREFIX + const_CORE_SUFFIX)
            self.db_plugin_iface = dbus.Interface(self.db_object,
                                dbus_interface=const_INT_PREFIX + const_PLUGIN_SUFFIX)
        except dbus.exceptions.DBusException,e:
            if e._dbus_error_name=='org.freedesktop.DBus.Error.ServiceUnknown':
                raise BridgeExceptionNoService
            else:
                raise e
        #props = self.db_core_iface.getProperties()

    def register(self, functionName, handler, iface="core"):
        if iface == "core":
            self.db_core_iface.connect_to_signal(functionName, handler)
        elif iface == "plugin":
            self.db_plugin_iface.connect_to_signal(functionName, handler)
        else:
            error(_('Unknown interface'))

##METHODS_PART##

#methods from plugins
    def getRoomJoined(self, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.getRoomJoined(profile_key)

    def getRoomSubjects(self, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.getRoomSubjects(profile_key)

    def joinMUC(self, service, roomId, nick, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.joinMUC(service, roomId, nick, profile_key)

    def tarotGameLaunch(self, players, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.tarotGameLaunch(players, profile_key)
    
    def tarotGameCreate(self, room_jid, players, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.tarotGameCreate(room_jid, players, profile_key)

    def tarotGameReady(self, player, referee, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.tarotGameReady(player, referee, profile_key)

    def tarotGameContratChoosed(self, player, referee, contrat, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.tarotGameContratChoosed(player, referee, contrat, profile_key)

    def tarotGamePlayCards(self, player, referee, cards, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.tarotGamePlayCards(player, referee, cards, profile_key)

    def quizGameLaunch(self, players, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.quizGameLaunch(players, profile_key)
    
    def quizGameCreate(self, room_jid, players, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.quizGameCreate(room_jid, players, profile_key)

    def quizGameReady(self, player, referee, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.quizGameReady(player, referee, profile_key)

    def quizGameAnswer(self, player, referee, answer, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.quizGameAnswer(player, referee, answer, profile_key)

    def sendFile(self, to, path, data, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.sendFile(to, path, data, profile_key)

    def findGateways(self, target, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.findGateways(target, profile_key)

    def getCard(self, target, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.getCard(target, profile_key)

    def getCardCache(self, target):
        return self.db_plugin_iface.getCardCache(target)

    def getAvatarFile(self, hash):
        return self.db_plugin_iface.getAvatarFile(hash)

    def in_band_register(self, target, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.in_band_register(target, profile_key)

    def gatewayRegister(self, action, target, data, profile_key='@DEFAULT@'):
        if data == None:
            data = [('', '')] #XXX: we have to do this awful hack because python dbus need to guess the signature
        return self.db_plugin_iface.gatewayRegister(action, target, data, profile_key)

    def getLastMicroblogs(self, jid, max_items, profile_key='@DEFAULT@', callback=None, errback=None):
        return self.db_plugin_iface.getLastMicroblogs(jid, max_items, profile_key, reply_handler=callback, error_handler=errback)

    def getMblogNodes(self, profile_key='@DEFAULT@', callback=None, errback=None):
        return self.db_plugin_iface.getMblogNodes(profile_key, reply_handler=callback, error_handler=errback)
    
    def sendGroupBlog(self, groups, message, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.sendGroupBlog(groups, message, profile_key)
    
    def sendPersonalEvent(self, event_type, data, profile_key='@DEFAULT@'):
        return self.db_plugin_iface.sendPersonalEvent(event_type, data, profile_key)
    
    def setMicroblogAccess(self, access="presence", profile_key='@DEFAULT@', callback=None, errback=None):
        return self.db_plugin_iface.setMicroblogAccess(access, profile_key, reply_handler=callback, error_handler=errback)

